<?php

/**
 * @file
 * Contains \Drupal\linkicon\Plugin\field\formatter\LinkIconFormatter.
 */

namespace Drupal\linkicon\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;
use Drupal\Core\Template\Attribute;
use Drupal\Component\Utility\String;

/**
 * Plugin implementation of the 'linkicon' formatter.
 *
 * @FieldFormatter(
 *   id = "linkicon",
 *   label = @Translation("Link icon, based on title"),
 *   field_types = {
 *     "link"
 *   },
 *   settings = {
 *     "rel" = "",
 *     "target" = "",
 *     "linkicon_prefix" = "icon",
 *     "linkicon_wrapper_class" = "",
 *     "linkicon_load" = 0,
 *     "linkicon_vertical" = 0,
 *     "linkicon_style" = "",
 *     "linkicon_color" = "",
 *     "linkicon_tooltip" = 0,
 *     "linkicon_no_text" = 0,
 *     "linkicon_position" = "",
 *     "linkicon_global_title" = "",
 *     "linkicon_size" = "",
 *     "linkicon_bundle" = "",
 *     "linkicon_font" = ""
 *   }
 * )
 *
 * @todo:
 * - https://drupal.org/node/2136197
 * - https://drupal.org/node/2005434
 */
class LinkIconFormatter extends LinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, array &$form_state) {
    $path     = drupal_get_path('module', 'linkicon');
    $elements = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    // Predefined titles are supposed to be controlled.
    // @todo recheck, we don't declare/set it but it is still there.
    if (isset($elements['trim_length'])) {
      unset($elements['trim_length']);
    }

    $elements['linkicon_prefix'] = array(
      '#type' => 'textfield',
      '#title' => t('Icon prefix class'),
      '#description' => t('A "prefix" or "namespace", e.g.: icon (Fontello), fa (FontAwesome), st-icon (Stackicons), genericon, fonticon, etc. <br>If the link title is <em>Facebook</em>, it will create classes: <em>icon icon-facebook</em> for Fontello, or <em>fa fa-facebook</em> for FontAwesome > 3. <br>The individual icon class itself is based on the link text key matching the pattern: icon-KEY, or fa-KEY.'),
      '#default_value' => $settings['linkicon_prefix'],
      '#required' => TRUE,
    );

    $elements['linkicon_wrapper_class'] = array(
      '#type' => 'textfield',
      '#title' => t('Additional wrapper class'),
      '#description' => t('Additional wrapper class for the entire icon list.'),
      '#default_value' => $settings['linkicon_wrapper_class'],
    );

    $elements['linkicon_load'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow linkicon to provide CSS assets'),
      '#default_value' => $settings['linkicon_load'],
      '#description' => t('Otherwise, DIY accordingly.'),
    );

    $elements['linkicon_vertical'] = array(
      '#type' => 'checkbox',
      '#title' => t('Vertical'),
      '#default_value' => $settings['linkicon_vertical'],
      '#description' => t('By default, icons are displayed inline. Check to make icons stacked vertically.'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_style'] = array(
      '#type' => 'select',
      '#title' => t('Icon style'),
      '#default_value' => $settings['linkicon_style'],
      '#options' => array(
        'round'    => t('Round'),
        'round-2'  => t('Round 2'),
        'round-5'  => t('Round 5'),
        'round-8'  => t('Round 8'),
        'round-10' => t('Round 10'),
        'square'   => t('Square'),
        'button'   => t('Button'),
      ),
      '#empty_option' => t('- None -'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
      '#description' => t('Button is more prominent if the title text is not hidden over the background color.'),
    );

    $elements['linkicon_color'] = array(
      '#type' => 'select',
      '#title' => t('Icon background color'),
      '#default_value' => $settings['linkicon_color'],
      '#options' => array(
        'grey'   => t('Grey'),
        'dark'   => t('Dark'),
        'purple' => t('Purple'),
        'orange' => t('Orange'),
        'blue'   => t('Blue'),
        'lime'   => t('Lime'),
        'red'    => t('Red'),
      ),
      '#empty_option' => t('- None -'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
      '#description' => t('Basic background color. You should do proper theming to suit your design better, and disable all this.'),
    );

    $elements['linkicon_tooltip'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display title as tooltip'),
      '#default_value' => $settings['linkicon_tooltip'],
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_no_text'] = array(
      '#type' => 'checkbox',
      '#title' => t('Visually hide the title text'),
      '#default_value' => $settings['linkicon_no_text'],
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_global_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Override title with a generic link title'),
      '#description' => t('If provided, the link title will be overriden with this text, e.g.: Visit the site, View Demo.'),
      '#default_value' => $settings['linkicon_global_title'],
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_tooltip"]' => array('checked' => FALSE),
          ':input[name*="linkicon_no_text"]' => array('checked' => FALSE),
        ),
      ),
    );

    $elements['linkicon_position'] = array(
      '#type' => 'select',
      '#title' => t('Icon position to the title text.'),
      '#default_value' => $settings['linkicon_position'],
      '#description' => t('By default icon is before the text.'),
      '#options' => array(
        'bottom' => t('Bottom'),
        'right'  => t('Right'),
        'top'      => t('Top'),
      ),
      '#empty_option' => t('Left'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
          ':input[name*="linkicon_no_text"]' => array('checked' => FALSE),
        ),
      ),
    );

    $icon_sizes = array(
      'small' => t('Small'),
      'medium' => t('Medium'),
      'large' => t('Large'),
      'xlarge' => t('X-large'),
      'xxlarge' => t('Xx-large'),
    );

    $elements['linkicon_size'] = array(
      '#type' => 'select',
      '#title' => t('Icon font size'),
      '#default_value' => $settings['linkicon_size'],
      '#options' => $icon_sizes,
      '#empty_option' => t('Default'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_font'] = array(
      '#type' => 'textfield',
      '#title' => t('Path to font icon CSS file'),
      '#description' => t('Valid path to CSS file, e.g.: sites/all/libraries/fontello/css/fontello.css. <br>Leave blank if your theme has already had one. <br>Overriden if Icon API preferred, or if the prefix matches one, e.g.: fontawesome.module. <br>Please be aware of possible conflicts if you installed different icon fonts <br>due to their namespace collision. Most icon fonts tend to use make use the same .icon:before.'),
      '#default_value' => $settings['linkicon_font'],
      '#element_validate' => array(array($this, 'validateLinkiconFontPath')),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    // Build a preview.
    if (function_exists('icon_bundles')) {
      $bundles   = icon_bundles();
      $providers = icon_providers();

      $provider_options = array();
      foreach ($providers as $name => $provider) {
        $provider_options[$provider['name']] = $provider['title'];
      }

      if ($provider_options) {
        $elements['linkicon_bundle'] = array(
          '#type' => 'select',
          '#title' => t('Icon module integration'),
          '#options' => $provider_options,
          '#default_value' => $settings['linkicon_bundle'],
          '#description' => t('The above icon providers modules are detected. You can choose which icon CSS file to load with this module. <br>Make sure that you have a working module that loads relevant CSS accordingly. <br>Known working modules as of this writing: fontawesome, and icomoon.'),
          '#empty_option' => t('- None -'),
        );
      }
    }

    if (!empty($settings['linkicon_bundle']) || !empty($settings['linkicon_font'])) {
      $icon_previews = array();
      $linkicon_item = array(
        '#theme' => 'linkicon_item',
        '#position' => $settings['linkicon_position'],
        '#title' => 'Twitter',
        '#attributes' => array('class' => array($settings['linkicon_prefix'], $settings['linkicon_prefix'] . '-twitter')),
        '#icon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : FALSE,
        '#icon_name'   => 'twitter',
      );

      $icon = drupal_render($linkicon_item);

      $tooltip = '';
      if ($settings['linkicon_tooltip']) {
        $tooltip = ' data-title="Twitter"';
      }

      foreach ($icon_sizes as $key => $size) {
        $is_active = $key == $settings['linkicon_size'] ? ' active' : '';
        $icon_previews[] = '<a class="linkicon__item linkicon--' . $key . $is_active . '" href="#"' . $tooltip . '>' . $icon . '</a>';
      }

      $preview = array(
        '#theme' => 'linkicon',
        '#linkicon_id' => 'linkicon-preview',
        '#items' => $icon_previews,
        '#config' => array(
          'style' => $settings['linkicon_style'],
          'color' => $settings['linkicon_color'],
          // 'size' => $settings['linkicon_size'],
          'no_text' => $settings['linkicon_no_text'],
          'position' => $settings['linkicon_position'],
          'tooltip' => $settings['linkicon_tooltip'],
          // 'vertical' => $settings['linkicon_vertical'],
          'wrapper_class' => $settings['linkicon_wrapper_class'],
          'load' => $settings['linkicon_load'],
        ),
      );

      if ($settings['linkicon_load']) {
        $preview['#attached']['css'][] = array('data' => $path . '/css/linkicon.css');
        if ($settings['linkicon_font'] && empty($settings['linkicon_bundle'])) {
          $preview['#attached']['css'][] = array('data' => $settings['linkicon_font']);
        }
      }

      $elements['linkicon_size_preview'] = array(
        '#markup' => render($preview),
      );
    }

    return $elements;
  }

  /**
   * Validate path to icon font CSS file.
   */
  public function validateLinkiconFontPath(array $element, array &$form_state) {
    if (!empty($element['#value']) && !is_file($element['#value'])) {
      form_error($element, $form_state, t('<strong>@css_font_path</strong> doesn\'t exist.', array('@css_font_path' => $element['#value'])));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $settings = $this->getSettings();

    if (!empty($settings['rel'])) {
      $summary[] = t('Add rel="@rel"', array('@rel' => $settings['rel']));
    }
    if (!empty($settings['target'])) {
      $summary[] = t('Open link in new window');
    }

    $summary[] = t('Base class: <em>@linkicon_prefix</em>.', array(
    '@linkicon_prefix' => $settings['linkicon_prefix'],
    ));

    $summary[] = t('Module CSS: <em>@linkicon_load</em>. Wrapper: <em>@linkicon_wrapper_class</em>. Style: <em>@linkicon_style</em>. Bg: <em>@linkicon_color</em>.', array(
      '@linkicon_load' => $settings['linkicon_load'] ? t('Yes') : t('No'),
      '@linkicon_wrapper_class' => $settings['linkicon_wrapper_class'] ? $settings['linkicon_wrapper_class'] : t('None'),
      '@linkicon_vertical' => $settings['linkicon_vertical'] ? t('Vertical') : t('Horizontal'),
      '@linkicon_style' => $settings['linkicon_style'] ? $settings['linkicon_style'] : t('None'),
      '@linkicon_color' => $settings['linkicon_color'] ? $settings['linkicon_color'] : t('None'),
    ));

    if ($settings['linkicon_load']) {
      $summary[] = t('Size: <em>@linkicon_size</em>. No text: <em>@linkicon_no_text</em>. Tooltip: <em>@linkicon_tooltip</em>', array(
        '@linkicon_size' => $settings['linkicon_size'],
        '@linkicon_no_text' => $settings['linkicon_no_text'] ? t('Yes') : t('No'),
        '@linkicon_tooltip' => $settings['linkicon_tooltip'] ? t('Yes') : t('No'),
      ));

      if (empty($settings['linkicon_no_text'])) {
        $summary[] = t('Use global title: <em>@linkicon_global_title</em>. <br>Icon position: <em>@linkicon_position</em>.', array(
          '@linkicon_global_title' => $settings['linkicon_global_title'] ? $settings['linkicon_global_title'] : t('No'),
          '@linkicon_position' => $settings['linkicon_position'] ? $settings['linkicon_position'] : t('Left'),
        ));
      }

      $summary[] = t('Font icon path: <em>@linkicon_font</em>.', array(
        '@linkicon_font' => $settings['linkicon_font'] ? $settings['linkicon_font'] : t('No'),
      ));

      // @todo icon API integration, none by now 3/3/14
      //$summary[] = t('Icon provider: <em>@linkicon_bundle</em>.', array(
        //'@linkicon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : t('No'),
      //));
    }
    // Must be array, unlike D7.
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareView(array $entities_items) {
    if ($this->getFieldSetting('title') == 'predefined') {
      $settings = $this->getFieldSettings();
      if (!empty($settings['title_predefined'])) {
        $title_predefined = linkicon_extract_allowed_values($settings['title_predefined']);
        $title_tooltip    = linkicon_extract_allowed_values($settings['title_predefined'], TRUE);

        foreach ($entities_items as $entity_id => $items) {
          $new_values = array();
          foreach ($items as $delta => $item) {
            $values = $item->getValue();
            $new_values['display_title'] = $title_predefined[$values['title']];
            if (isset($title_tooltip[$values['title']])) {
              $new_values['tooltip'] = $title_tooltip[$values['title']];
            }
            $merged_values = array_merge($values, $new_values);
            $item->setValue($merged_values);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items) {
    $path         = drupal_get_path('module', 'linkicon');
    $element      = array();
    $contents     = array();
    $entity       = $items->getEntity();
    $entity_type  = $entity->getEntityTypeId();
    $bundle       = $entity->bundle();
    $id           = $entity->id();
    $settings     = $this->getSettings();
    $field_name   = $this->fieldDefinition->getName();

    foreach ($items as $delta => $item) {
      $attributes = array();
      $attributes['class'][] = 'linkicon__item';

      // Linkicon requires both link text and URL available with proper
      // validation during input, no need extra checks.
      // Unsanitized token replacement here because $options['html'] is FALSE
      // by default in l().
      $icon_name     = \Drupal::token()->replace($item->title, array($entity_type => $entity), array('sanitize' => FALSE, 'clear' => TRUE));
      $display_title = isset($item->display_title) ? $item->display_title : $icon_name;
      $tooltip       = isset($item->tooltip) ? $item->tooltip : $display_title;
      $link          = $this->buildLink($item);
      // @todo recheck if any icon class has any silly Uppercase.
      $icon_class    = drupal_clean_css_identifier(drupal_strtolower($settings['linkicon_prefix'] . '-' . $icon_name));

      // If title is overriden with a generic one, make sure the $icon_class
      // is not overridden.
      if (!empty($settings['linkicon_title_global']) && empty($settings['linkicon_no_text'])) {
        $display_title = $settings['linkicon_title_global'];
      }

      $link['options']['html'] = TRUE;
      $icon_element = array(
        '#theme'       => 'linkicon_item',
        '#position'    => $settings['linkicon_position'],
        '#title'       => String::checkPlain($display_title),
        '#attributes'  => array('class' => array($settings['linkicon_prefix'], $icon_class)),
        '#icon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : FALSE,
        '#icon_name'   => $icon_name,
      );

      // Our pure CSS3 tooltip depends on data-title.
      if ($settings['linkicon_tooltip']) {
        $attributes['data-title'] = String::checkPlain($tooltip);
      }

      // Merge with core options: rel and target.
      $link['options']['attributes'] += $attributes;

      // We are done, pass it over to link to do its job.
      $contents[$delta] = array(
        '#type' => 'link',
        '#title' => $icon_element,
        '#href' => $link['path'],
        '#options' => $link['options'],
      );
    }

    // Build own wrapper for greater control.
    $linkicon_id = drupal_clean_css_identifier("{$entity_type}-{$bundle}-{$field_name}-{$id}");
    $element[0] = array(
      '#theme' => 'linkicon',
      '#linkicon_id' => 'linkicon-' . $linkicon_id,
      '#items' => $contents,
      '#config' => array(
        'style'         => $settings['linkicon_style'],
        'color'         => $settings['linkicon_color'],
        'size'          => $settings['linkicon_size'],
        'no_text'       => $settings['linkicon_no_text'],
        'position'      => $settings['linkicon_position'],
        'tooltip'       => $settings['linkicon_tooltip'],
        'vertical'      => $settings['linkicon_vertical'],
        'wrapper_class' => $settings['linkicon_wrapper_class'],
        'load'          => $settings['linkicon_load'],
      ),
    );

    // Attached our assests if so configured.
    if ($settings['linkicon_load']) {
      $element[0]['#attached']['css'][] = array('data' => $path . '/css/linkicon.css');
      if ($settings['linkicon_font'] && empty($settings['linkicon_bundle'])) {
        $element[0]['#attached']['css'][] = array('data' => strip_tags($settings['linkicon_font']));
      }
    }

    return $element;
  }
}
